(define-module (ui rd_xdmcp ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator card-index)
               :export (init))

; Debug string (paste in for your needs).
;(format #t "XXX: ~S~%" rdp_profile_name)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CREATE PAGE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Create profile form
(define (create-new-profile)
  (catch/message
    (lambda()
      (hide-all)
      ; Show form
      (form-update-visibility "new_profile_invalid" #f)
      (form-update-visibility '("new_profile_area") #t)
      ; Reset protocol name
      (form-update-value "new_profile_name"  ""))))


; Create new profile based on entered values
(define (create-new-profile-confirm)
  (let
    ((name (form-value "new_profile_name")))
    ; Test for correctness input	
    (if (regexp-exec (make-regexp (string-append "^" "" "$") regexp/extended) name)
      ; Empty name
      (and
        (form-update-value "new_profile_invalid_reason" "Profile name can't be empty")
        (form-update-visibility "new_profile_invalid" #t))
      ; Good name
      (and (catch/message
             (lambda () (woo-new "/rd"
                                 'name name
                                 'protocol "xdmcp")))
           (form-update-visibility '("new_profile_area") #f)
           (select-profile name)))))

; Select just created profile
(define (select-profile name)
  (form-update-value "avail_profiles" name)
  (edit-profile))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; XDMCP
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; XDMCP init
(define (xdmcp-init)
  (form-value "name")
  (form-bind "xdmcp_btn_reset" "click" edit-profile)
  (form-bind "xdmcp_btn_save"  "click" xdmcp-write)
  ; Selection profile lead to it editing
  (form-bind "xdmcp_connection_method" "change" connection-method-changed))

; Change visibility some widgets
(define (connection-method-changed)
	(let* ((method (f-get-ci-value "xdmcp_connection_method")))
		(cond 
          ((string=? method "direct") (form-update-activity "xdmcp_server" #t))
          ((string=? method "browse") (form-update-activity "xdmcp_server" #t))
          (else (form-update-activity "xdmcp_server" #f)))))

; Save XDMCP profile
(define (xdmcp-write)
  (catch/message (lambda()
                   (apply woo "xdmcp" "/rd_xdmcp/write"
                          (form-value-list xdmcp-parameters))))
  (edit-profile))

; XDMCP bools
(define xdmcp-bools '(
                      "xdmcp_once"
                      "xdmcp_fullscreen"
                      ))

; XDMCP labels
(define xdmcp-labels '(
                       "xdmcp_profile_name"
                       ))

; XDMCP inputs
(define xdmcp-inputs '(
                       "xdmcp_server"
                       ))

; XDMCP lists
(define xdmcp-lists '(
                      "xdmcp_connection_method"
                      "xdmcp_geometry"
                      "xdmcp_color_depth"
                      ))

; Known XDMCP parameter
(define xdmcp-parameters (append '("avail_profiles")
                           (append xdmcp-bools
                           (append xdmcp-labels
                           (append xdmcp-inputs
                           xdmcp-lists)))))

; Fill wigets with default parameters
(define (set-default-parameters)
  (form-update-value "xdmcp_connection_method" "broadcast")
  (form-update-value "xdmcp_once" #f)
  (form-update-value "xdmcp_fullscreen" #f)
  )

; Edit XDMCP
(define (xdmcp-edit xdmcp_profile_name)

	(and

	; Reset all values
	(for-each (lambda(p) (form-update-value p "")) xdmcp-parameters)

	; Select profile in profiles list
	(form-update-value "avail_profiles" xdmcp_profile_name)

	; Fill all lists
	(for-each (lambda(lst) 
							(form-update-enum lst (woo "xdmcp" (string-append "/rd_xdmcp/list_" lst) 'xdmcp_profile_name xdmcp_profile_name 'language (form-value "language")))) xdmcp-lists)

	; Set default parameters
	(set-default-parameters)

	; Read current settings
	(let* ((profile_info (woo "xdmcp" "/rd_xdmcp/read" 'xdmcp_profile_name xdmcp_profile_name 'language (form-value "language"))))
		(form-update-value-list xdmcp-parameters profile_info))

	; Update visibility same widgets
	(connection-method-changed)

	; Turn form visibility
	(form-update-visibility '("xdmcp_area") #t)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN MODULE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Confirm delete selected profile
(define (delete-profile-confirm)
  (hide-all)
  (let ((profile_name (f-get-ci-value "avail_profiles")))
    (form-update-value "delete_profile_name" profile_name)
    ; Show form
    (form-update-visibility '("delete_profile_area") #t)))
           

; Delete profile
(define (delete-profile)
  (let ((profile_name (f-get-ci-value "avail_profiles")))
    (and 
      (catch/message (lambda() (woo-delete "/rd" 'profile_name profile_name 'language (form-value "language"))))
      (init-interface))))

; Interface initialization
(define (init-interface) 
  (catch/message 
    (lambda()
      ; Hide all modules
      (hide-all)
      ; Ask for available profiles
      (form-update-enum "avail_profiles" (woo-list "/rd_xdmcp/avail_profiles" 'language (form-value "language"))))))

; Hide all
(define (hide-all)
  (form-update-visibility '("new_profile_area" "delete_profile_area" "delete_button" "xdmcp_area") #f))

; Edit profile with specific type
(define (edit-profile)
  ; Update list, keep previous value or set the first one, return value
  (f-update-ci-list "avail_profiles" "/rd_xdmcp/avail_profiles")
  ; Get profile info
  (let* ((profile_name (f-get-ci-value "avail_profiles")))
    (hide-all)
    (form-update-visibility '("delete_button") #t)
    (xdmcp-edit profile_name)))

; Initialization routine
(define (init)
  ; Interconnect button with action 
  (form-bind "button_create_new_profile" "click" create-new-profile)
  (form-bind "button_create_new_profile_confirm" "click" create-new-profile-confirm)
  (form-bind "delete_button" "click" delete-profile-confirm)
  (form-bind "delete_profile_confirm" "click" delete-profile)
  (form-bind "delete_profile_cancel" "click" edit-profile)
  ; Selection profile lead to it editing
  (form-bind "avail_profiles" "change" edit-profile)
  ; Interface initialization
  (init-interface)
  ; Init modules
  (xdmcp-init))
